package MOBY::RDF::Predicates::RDF;

use strict;
BEGIN {

use constant RDF_PREFIX => 'rdf';

use constant RDF_URI => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#';

################################
## Predicates for RDF         ##
################################

use constant type => RDF_URI . 'type';


}
1;
